#!/usr/bin/python

from math import *
import numpy as np
import Manyo as m
import NewLevmar as Adv
#import Adv
import time

####    linear combination of gaussian    ####
#def dist(x, n, a, c, w):
def dist(x, param):
    sum=param[0]
    for i in range(len(param[1])):
        a, c, w=param[1][i]
        sum = sum + a*exp(-1.0*((x-c)/w)**2)
    return sum

####    sign function    ####
def sign(r):
    if r > 0.0:
        s = 1.0
    elif r == 0.0:
        s = 0.0
    else:
        s = -1.0
    return s

####    initialize an ElementContainer    ####
#def initElementContainer(xmin, xmax, nDiv):
def initElementContainer(xmin, xmax, nDiv, param):

    bin = m.MakeDoubleVector(nDiv+1)
    delta = (xmax - xmin)/nDiv
    for i in range(nDiv+1):
        bin[i] = xmin + delta*i

    xc = m.MakeDoubleVector(nDiv)
    for i in range(nDiv):
        xc[i] = (bin[i] + bin[i+1])/2.0

    #nPeak=1
    #a=m.MakeDoubleVector(nPeak)
    #c=m.MakeDoubleVector(nPeak)
    #w=m.MakeDoubleVector(nPeak)
    #a[0]=100000.0
    #c[0]=7.5
    #w[0]=2.5
    y = m.MakeDoubleVector(nDiv)
    e = m.MakeDoubleVector(nDiv)
    for i in range(nDiv):
        #p = dist(xc[i], nPeak, a, c, w)
        p = dist(xc[i], param)
        q = sign(np.random.rand(1)-0.5)*(p/10.0)*sqrt(-1.0*log(np.random.rand(1)))
        y[i] = p + q
        e[i] = p/10.0

    ec = m.ElementContainer()
    ec.AddToHeader("run number", 1)
    ec.AddToHeader("level",      1)
    ec.AddToHeader("Inst.",      "manyo")
    ec.Add("TOF",       bin, "sec." )
    ec.Add("Intensity", y,   "count")
    ec.Add("Error",     e,   "count")
    ec.SetKeys("TOF", "Intensity", "Error")

    return ec

def outputElementContainer(ec):
    bin = ec.Put(ec.PutXKey())
    y   = ec.Put(ec.PutYKey())
    e   = ec.Put(ec.PutEKey())

    print str("No.").rjust(4), str("----------- bin -----------").rjust(27), str("xc").rjust(10), str("Intensity").rjust(23), str("error").rjust(23)
    for i in range(ec.PutSize(ec.PutYKey())):
         print str(i).rjust(4), "[", str(bin[i]).rjust(10), ",", str(bin[i+1]).rjust(10), ")", str((bin[i]+bin[i+1])/2.0).rjust(10), str(y[i]).rjust(23), str(e[i]).rjust(23)

####    plot    ####
def knownTerminal(type):
    retval=False
    if type=="emf":
        retval=True
    elif type=="jpeg":
        retval=True
    elif type=="png":
        retval=True
    elif type=="postscript":
        retval=True
    elif type=="x11":
        retval=True
    else:
        if type == "":
            print "knownTerminal: empty string interpreted x11\n"
        else:
            print "knownTerminal: unknown terminal type: ", type
    print "knownTerminal: ", retval
    return retval

def createTerminalCmd(terminalType):
    cmd=""
    if terminalType == "emf":
        cmd="set terminal emf"
    elif terminalType == "jpeg":
        cmd="set terminal jpeg"
    elif terminalType == "png":
        cmd="set terminal png"
    elif terminalType == "postscript":
        cmd="set terminal postscript eps"
    elif terminalType == "x11":
        cmd=""
    else:
        if terminalType != "":
            print "createTerminalCmd: unknown terminal type: ", type
    print "createTerminalCmd: ", cmd
    return cmd


def createOutputCmd(terminalType, fileNameStem):
    cmd=""
    if terminalType == "emf":
        cmd="set output '"+fileNameStem+".emf"  + "'"
    elif terminalType == "jpeg":
        cmd="set output '"+fileNameStem+".jpeg" + "'"
    elif terminalType == "png":
        cmd="set output '"+fileNameStem+".png"  + "'"
    elif terminalType == "postscript":
        cmd="set output '"+fileNameStem+".eps"  + "'"
    elif terminalType == "x11":
        cmd=""
    else:
        if terminalType != "":
            print "createOutoutCmd: unknown terminal type: ", type
    print "createOutputCmd: ", cmd
    return cmd

def plotPeakData(src, smoothed, peakData, terminal):
    srcBin=src.Put(src.PutXKey())

    g=m.GnuplotInterface()

    if knownTerminal(terminal):
        g.e(createTerminalCmd(terminal))
        g.e(createOutputCmd(terminal, "peakData"))

    g.e("set xlabel '" + src.PutXKey() + " / " + src.PutUnit(src.PutXKey()) + "'")
    g.e("set xrange[" + str(srcBin.front()) + ":" +str(srcBin.back())+"]")

    g.e("set ylabel '" + src.PutYKey() + " / " + src.PutUnit(src.PutYKey()) + "'")
    g.e("set autoscale y")

    v=peakData.toVector()
    for i in range(peakData.size()):
        peak=peakData.getPeak(i)
        v=peak.toFullVector()
        h=v[0] # height
        c=v[1] # position
        l=v[3] # left  pos at HWHM
        u=v[4] # right pos at HWHM
        #print "peak id = ", i, p
        g.e("set arrow "+str(3*i+1)+" from "+str(c)+", 0 to "+str(c)+", 150000 nohead\n")  # guide line for a peak or a sholder
        g.e("set label "+str(4*i+1)+" '"+str(c)+"' at "+str(c)+",10000" )                  # the position of a peak or a sholder
        g.e("set label "+str(4*i+2)+" '"+str(h)+"' at "+str(c)+","+str(h)+" front" )       # the height   of a peak or a sholder
        if c-l > 0.0:
            g.e("set arrow "+str(3*i+2)+" from "+str(c)+","+str(h/2.0)+" to "+str(l)+","+str(h/2.0)) # guide line for the lower bound at hwhm
            g.e("set label "+str(4*i+3)+" '"+str(c-l)+"' at "+str((c+l)/2.0)+","+str(h/2.0)+" center front" )  # the width
        if u-c > 0.0:
            g.e("set arrow "+str(3*i+3)+" from "+str(c)+","+str(h/2.0)+" to "+str(u)+","+str(h/2.0)) # guide line  for the upper bound at hwhm
            g.e("set label "+str(4*i+4)+" '"+str(u-c)+"' at "+str((c+u)/2.0)+","+str(h/2.0)+" center front" )  # the width

    while 1:
        g.e("plot '-' using 1:2 with lines linewidth 1 title 'source', '-' using 1:3 with lines linewidth 1 title 'smoothed'")

        for i in range(src.PutSize(src.PutYKey())):
            g.e( str( (srcBin[i]+srcBin[i+1])/2.0 ) + " " + str( src.Put(src.PutYKey(), i) )+ " " + str( smoothed.Put(smoothed.PutYKey(), i) ) )
        g.e("e")

        if terminal=="x11" or terminal=="":
            c=raw_input("hit any key except return for next\n")
            if c != "":
                break
        else:
            g.e("replot")
            break
    #g.e("pause -1 'hit any key'")

    del g
    del srcBin


def plotNorm(method, terminal):
    ic = method.getConvergenceHistoryForInt4(  Adv.NewLevmar.ITERATION_COUNT  )
    rf = method.getConvergenceHistoryForDouble(Adv.NewLevmar.R_FACTOR         )
    rn = method.getConvergenceHistoryForDouble(Adv.NewLevmar.RESIDUAL_ERR_NORM)
    gn = method.getConvergenceHistoryForDouble(Adv.NewLevmar.GRADIENT_NORM    )
    pn = method.getConvergenceHistoryForDouble(Adv.NewLevmar.PARAM_DIFF_NORM  )
    waitTime  = method.getConvergenceHistoryForInt4(  Adv.NewLevmar.ITERATION_TIME ).back()

    g=m.GnuplotInterface()

    #if knownTerminal(terminal):
    #    g.e(createTerminalCmd(terminal))
    #    g.e(createOutputCmd(terminal, "norm"))

    g.e("set xlabel 'iteration'")
    if ic.size() == 1:
        cmd="set xrange [-0.5:0.5]"
    else:
        cmd="set xrange ["+repr(ic.front())+":"+repr(ic.back())+"]"
    g.e(cmd)

    g.e("set ylabel 'norm'")
    g.e("set autoscale y")
    g.e("set logscale y 10.0")

    cmd =  "plot"
    cmd = cmd + "  '-' using 1:2 with linespoints title '" + Adv.NewLevmar.R_FACTOR          + "'"
    #cmd = cmd + ", '-' using 1:3 with linespoints title '" + Adv.NewLevmar.RESIDUAL_ERR_NORM + "'"
    #cmd = cmd + ", '-' using 1:4 with linespoints title '" + Adv.NewLevmar.GRADIENT_NORM     + "'"
    #cmd = cmd + ", '-' using 1:5 with linespoints title '" + Adv.NewLevmar.PARAM_DIFF_NORM   + "'"
    g.e(cmd)

    for i in range(ic.size()):
        data =              str(ic[i]/1.0).rjust(5)
        data = data + " " + str(rf[i]    ).rjust(23)
        data = data + " " + str(rn[i]    ).rjust(23)
        data = data + " " + str(gn[i]    ).rjust(23)
        data = data + " " + str(pn[i]    ).rjust(23)
        g.e(data)
        #print data
    g.e("e")

    g.e("pause " + str(waitTime/1000000))

    del g
    del ic
    del rf
    del rn
    del gn
    del pn
    del waitTime


def plot(src, smoothed, fittied, terminal):
    srcBin      = src.Put(src.PutXKey())
    srcY        = src.Put(src.PutYKey())
    smoothedBin = smoothed.Put(smoothed.PutXKey())
    smoothedY   = smoothed.Put(smoothed.PutYKey())
    fittiedBin  = fittied.Put(fittied.PutXKey())
    fittiedY    = fittied.Put(fittied.PutYKey())

    g=m.GnuplotInterface()

    #if knownTerminal(terminal):
    #    g.e(createTerminalCmd(terminal))
    #    g.e(createOutputCmd(terminal, "result"))

    g.e("set xlabel '" + src.PutXKey() + " / " + src.PutUnit(src.PutXKey()) + "'")
    g.e("set xrange[" + str(srcBin.front()) + ":" +str(srcBin.back())+"]")

    g.e("set ylabel '" + src.PutYKey() + " / " + src.PutUnit(src.PutYKey()) + "'")
    g.e("set autoscale y")

    while 1:
        cmd="plot '-' using 1:2 with lines linewidth 1 title 'original', '-' using 1:3 with lines linewidth 2 title 'smoothed', '-' using 1:4 with lines linewidth 2 title 'fittied'"
        g.e(cmd)

        for i in range(srcY.size()):
            data=str( (srcBin[i]+srcBin[i+1])/2.0 ) + " " + str( srcY[i] )+ " " + str( smoothedY[i] ) + " " + str( fittiedY[i] )
            g.e(data)
        g.e("e")

        g.e("replot")
        c=raw_input("hit any key except return\n")
        if c != "":
            break

    del g
    del srcBin
    del srcY
    del fittiedBin
    del fittiedY

####    main routine    ####
xmin = 0.0
xmax = 15.0
nDiv = 1500
param1=[1000.0,     [[100000.0, 5.0, 1.5], [100000.0, 10.0, 1.5], [35000.0, 2.8, 1.0] ]]

src = initElementContainer(xmin, xmax, nDiv, param1)
#outputElementContainer(src)

peakSearch = Adv.PeakSearch(src, Adv.BSPLINE)
peakSearch.setParam(Adv.BSpline.NUMBER_OF_BREAK_POINTS, 16)
if peakSearch.checkParam():
    peakSearch.execute()
    smoothed = peakSearch.getResult()
    peakData = peakSearch.getPeaks()
    plotPeakData(src, smoothed, peakData, "postscript")
    plotPeakData(src, smoothed, peakData, "")

domain = Adv.Domain()
domain.setSource(src)
domain.setRange(xmin, xmax)
print "[", str(domain.getLowerBound()).rjust(10), ",", str(domain.getUpperBound()).rjust(10), "]"
print "[", str(domain.getLowerBoundID()).rjust(10), ",", str(domain.getUpperBoundID()).rjust(10), "]"
print
print "type of the domain", domain.getType()
#domain.setType(Domain.OO)
#print "type of the domain", domain.getType()

levmar=Adv.NewLevmar()

paramSet = levmar.setDefaultParam(src);
paramSet.dump()

# initial values of fitting parameters
param=peakData.toVector()

# box constrain for fitting parameters
lb=m.DoubleVector()
for i in range(param.size()):
    lb.push_back(param[i]*0.9)
ub=m.DoubleVector()
for i in range(param.size()):
    ub.push_back(param[i]*1.1)

paramSet.add(Adv.NewLevmar.PARAMETER_VALUES, param)
paramSet.add(Adv.NewLevmar.LOWER_BOUNDS,     lb)
paramSet.add(Adv.NewLevmar.UPPER_BOUNDS,     ub)

#combinations of fitting function
funcStr=""
for i in range(peakData.size()):
    funcStr=funcStr+" g"
   
parser=Adv.FuncParser(funcStr) # a string expression for list of function names or symbols
#funcList=parser.parse()
paramSet.add(Adv.NewLevmar.FUNCTIONS, parser.parse())

paramSet.replace(Adv.NewLevmar.CONSTRAIN,          Adv.NewLevmar.NO_CONSTRAIN)  # default Adv.NewLevmar.BOX
paramSet.replace(Adv.NewLevmar.USE_NUMERICAL_DIFF, False)                             # default True
#paramSet.replace(Adv.NewLevmar.DIFF_METHOD,        Adv.NewLevmar.CENTRAL)       # default FOWARD
#paramSet.replace(Adv.NewLevmar.USE_DATA_WEIGHTS,   False)                             # default True
paramSet.replace(Adv.NewLevmar.MAX_ITERATIONS,     1000)                              # default 1000
paramSet.replace(Adv.NewLevmar.OUTPUT_INTERVAL,    50)                               # default 50

paramSet.replace(Adv.NewLevmar.SCALING_FACTOR,     Adv.NewLevmar.DEFAULT_SCALING_FACTOR)     # default 0.001
paramSet.replace(Adv.NewLevmar.TOLERANCE,          Adv.NewLevmar.DEFAULT_TOLERANCE)          # default 1.0e-17
paramSet.replace(Adv.NewLevmar.GRADIENT_TOLERANCE, Adv.NewLevmar.DEFAULT_GRADIENT_TOLERANCE) # default 1.0e-17
paramSet.replace(Adv.NewLevmar.RELATIVE_TOLERANCE, Adv.NewLevmar.DEFAULT_RELATIVE_TOLERANCE) # default 1.0e-17
paramSet.dump()

print "parameter check: ", levmar.checkParam(src, domain, paramSet)  # parameter check, true if parametes are consistent

if  levmar.checkParam(src, domain, paramSet):
    levmar.toInnerForm(src, domain, paramSet)  # translate inner form
    levmar.fit()                               # fitting start

    if levmar.isMultiThreaded():               # if levmar is multi-threaded method, wait while fitting
        ct=-1
        while levmar.isFitting():              # loop until stopping function fit
            time.sleep(1)                      #     
            s = levmar.getHistorySize()
            if s > ct:
                #    print "ct=", ct, " hestory size=", s
                plotNorm(levmar, "")
                ct = s
        plotNorm(levmar, "")

    levmar.eval()                              # evaluate values of functions useing fitted parameter
    fitted=m.ElementContainer()
    levmar.toElementContainer(src, fitted)     # copy fitted results to a element container
    plot(src, smoothed , fitted, "")
    #outputElementContainer(fitted)




from __future__ import print_function
import os
import datetime
import Manyo as mm
import Manyo.Utsusemi as mu
import time as dtime
import uGao.MPlot as mp
filename = 'Log_TC_'

#def CheckSampleTemp(date1="2012-04-02",start_time="14:15:00",date2="2012-04-02",end_time="16:15:00",thermoType=1): T. Yamada comment out [121228]
def CheckSampleTemp(date1="2012-04-02",start_time="14:15:00",date2="2012-04-02",end_time="16:15:00",tempType="Si",ECOutPut=True): 
     """
         Check Sample Temperature by using LakeShore-340
         @param date1      (string) start date
         @param start_time (string) start time
         @param date2      (string) end date
         @param end_time   (string) end time
         @param thermoType   (string) select temperature channel (A, B, C, C2...)
         @param ECOutPut   (bool) output EC (True) or Meanvalue (False)
         @retval result    (array)
     """

     ####################################################################################################
     #
     #date1="2012-04-02"
     #date2="2012-04-02"

#    param = 43
     Time = date1 +" "+start_time
     st0 = datetime.datetime(*dtime.strptime(Time,"%Y-%m-%d %H:%M:%S")[0:6])
     dt1 = datetime.datetime(2012,0o5,1,0,0,0,)
     dt2 = datetime.datetime(2012,0o5,24,0,0,0,)
     dt3 = datetime.datetime(2012,0o6,0o5,0,0,0,)     #T. Yamada add[121129]

     if st0 < dt1:
        param = 43
     elif st0 > dt1 and st0 < dt2:
        param = 61
     elif st0 > dt2 and st0 < dt3:
        param = 61
    #T. Yamada add [121129]
     elif st0 > dt3:
        #param = 71
        param = 65 #T. Yamada Modified [121228]
    #T. Yamada end [121129]

     #T. Yamada add[121228]
     thermoType = GetthermoType(tempType) 
     #T.Yamada end[121228]
#     basepath = os.path.join(os.environ["HOME"],"advancesoft","miya_test")
     #basepath = os.path.join(os.environ["HOME"],"ana","tmp") #20141220[Yamada]
     basepath = os.path.join(mu.UtsusemiEnvGetUserDir(),"ana","tmp") #2023[Y.Inamura] 
     #basepath = os.path.join(os.environ["MLF_TOP_DIR"],"log","mw")


     ####################################################################################################

     # file open and connect

     start_time_list = start_time.split(":")
     end_time_list = end_time.split(":")

     filename1 = filename + date1 +".csv"
     filename2 = filename + date2 +".csv"

     filepath1 = os.path.join(basepath,filename1)
     filepath2 = os.path.join(basepath,filename2)

     print(filepath1)
     tmppath = os.path.join(basepath,"tmp")

     if date1 == date2:
          if os.path.exists(filepath1):
              os.system("cat " + filepath1 + " >" + tmppath)
          else:
              result_ec = mm.ElementContainer()
              result = []
              return result,result_ec
     else :
          if os.path.exists(filepath1) and os.path.exists(filepath2):          
              os.system("cat " + filepath1 + " " + filepath2+ " >" + tmppath)     
          else:
              result_ec = mm.ElementContainer()
              result = []
              return result,result_ec

     f = open(tmppath)
     lines = f.readlines()
     f.close()
     line_num = len(lines)

     list_array = []
     date_array = []
     time_array = []

     time_axis = []
    
     start_num=0
     end_num=0

     # search initial and final real time from log data
     ##### <- TI & TY [20130121]    
     start_time_h = datetime.datetime(*dtime.strptime((date1+" "+start_time),"%Y-%m-%d %H:%M:%S")[0:6])
     end_time_h = datetime.datetime(*dtime.strptime((date2+" "+end_time),"%Y-%m-%d %H:%M:%S")[0:6])
     start_time_h = start_time_h-datetime.timedelta(minutes=5)
     end_time_h = end_time_h+datetime.timedelta(minutes=10)

     temperatures = []

     for line in lines:
          infos = line.split(",")
          if infos[1] == "STATUS":
               tmain,tms = infos[0].split(".")
               date = datetime.datetime(*dtime.strptime(tmain,"%Y-%m-%d %H:%M:%S")[0:6])
               if date <= start_time_h:
                    pass
               elif date >= end_time_h:
                    break
               else:
                    date = date-start_time_h
                    #time_array.append(date.days*24*3600+date.seconds+float(tms)/1000) T.Yamada Modified [130226]
                    time_array.append(date.days*24*3600+date.seconds+float(tms)/1000-300)
                    list_array.append(infos)
     start_num = 0
     end_num = len(list_array)

     print("start_time_h", start_time_h)
     print("end_time_h", end_time_h) 
     start_day = 2+365*12

     # calclation of mean min max and so on

     start_val = checkStartEndVal(list_array,start_num,param,thermoType)
     end_val = checkStartEndVal(list_array,(end_num-1),param,thermoType)
     min_val,max_val,sum_val,time_integral = CalcVal(list_array,start_num,end_num,param,thermoType,time_array)

     # append calc result      

     result = []
     if abs(time_integral) < 0.00001:
           print("integral of data is zero ?? please check start-time and end time")
           print(time_integral) 
           for i in range(0,20):
                 result.append(0)
     else:
           result.append(float(sum_val/time_integral))
           result.append(float(min_val))
           result.append(float(max_val))
           result.append(float(start_val))
           result.append(float(end_val))

     print("TemperatureRead >> Temp[K]: mean=%.2f,\tmin=%.2f,\tmax=%.2f,\tstart=%.2f,\tend=%.2f"\
            %(result[0], result[1], result[2], result[3], result[4]))
  
     # put value to ElementContainer
     Xaxis = time_array
     Yaxis = []
     Eaxis = []

     if ECOutPut :
          if len(Xaxis)>2:
               for i in range(len(Xaxis)-1):
                    TM1 = checkStartEndVal(list_array,i,param,thermoType)
                    TM2 = checkStartEndVal(list_array,i+1,param,thermoType)
                    Yaxis.append((TM1+TM2)/2)
                    Eaxis.append(0)

               result_ec = mm.ElementContainer()
               result_ec.Add("Xaxis",Xaxis,"sec")
               result_ec.Add("Yaxis",Yaxis,"K")
               result_ec.Add("Eaxis",Eaxis)
               result_ec.SetKeys("Xaxis","Yaxis","Eaxis")

              #p = mp.MPlot(result_ec)
              #p.SetPlotParam(0,ls="-",lw=4,mk="",eb=False)
              #p.SetYLabel("Temperature")
              #p.SetXLabel("Time")
              #subTitle = date1+" "+start_time+" ~ "+date2+" "+end_time
              #p.SetSubTitle(subTitle)

               return result_ec
          else:
               result_ec = mm.ElementContainer()
               return result_ec
     else:
          return result
     ##### -> TI & TY [20130121]
######################################################################################

def DateArray2TimeArray(date_array,time_array,start_num,end_num,date1,date2):

     time_a =[]
     for i in range(start_num,end_num):
        if date_array[i][0] == date1:
            time_a.append((float(time_array[i][0])*60*60\
                          +float(time_array[i][1])*60\
                          +float(time_array[i][2]) )/(60*60*24))
        elif date_array[i][0] == date2:
            time_a.append(((float(time_array[i][0]) + 24.0)*60*60\
                           +float(time_array[i][1])*60\
                           +float(time_array[i][2]) )/(60*60*24))
        else:
            print("start-time and end-time infomation are not including in log-files")

     return time_a

######################################################################################

def checkStartEndVal(list_array,ii,param,thermoType):

     if(ii<len(list_array)):
          return_val = list_array[ii][param+(thermoType-1)*2]
     else:
          print("Check ii>", ii)
          reurn_val = 0

     if return_val=="": return_val1=0
     return float(return_val)

######################################################################################

def CalcVal(list_array,start_num,end_num,param,thermoType,time_array):

     sum_val = 0
     min_val = 10000
     max_val = 0
     time_integral = 0

     for i in range(start_num,end_num-1):
       if list_array[i][1] =="STATUS":

           if list_array[i][param+(thermoType-1)*2] == "":
               print("Check thermoType")              
               val = 0
           else:
               val =  float(list_array[i][param+(thermoType-1)*2])
               sum_val += val * (time_array[i+1] - time_array[i])
               time_integral += time_array[i-start_num+1] - time_array[i-start_num]
               if min_val>val:
                    min_val=val
               if max_val<val:
                    max_val=val

     return min_val,max_val,sum_val,time_integral

######################################################################################

def CalcTime(list_array,start_num,end_num,param,thermoType,time_a,start_day):

     time_array = []
     time_array_m = []
     error_array = []
     time_integral = 0

     for i in range(start_num,end_num-1):
       if list_array[i][1] =="STATUS":
           if list_array[i][param+(thermoType-1)*2] != "":
               time_integral += time_a[i-start_num+1] - time_a[i-start_num]

           # Time define
           time = (time_a[i-start_num] + start_day)
#           time_m = (time_a[i-start_num]-time_a[0])*60*60*24

           time_array.append(time)
#           time_array_m.append(time_m)
#           error_array.append(0)

#     return time_integral,time_array,time_array_m,error_array
     return time_integral

######################################################################################
#T. Yamada start [121228]
def GetthermoType(TCh="C2"):
    if(TCh=="A"):
        return 1
    elif(TCh=="B"):
        return 2
    elif(TCh=="C")or(TCh=="C1"):
        return 3
    elif(TCh=="C2")or(TCh=="Si"):
        return 4
    elif(TCh=="C3"):
        return 5
    elif(TCh=="C4"):
        return 6
    elif(TCh=="D")or(TCh=="D1"):
        return 7
    elif(TCh=="D2")or(TCh=="Pt"):
        return 8
    elif(TCh=="D3"):
        return 9
    elif(TCh=="D4"):
        return 10
    elif(int(TCh)>=1 and int(TCh)<=10):
        return int(TCh)
    else:
        return 4
#T. Yamada end [121228]
######################################################################################



from __future__ import print_function
import Tkinter
import tkFileDialog
import DiffractionGUIMain as DGM
import DiffractionBase as DB
import vis.DetectMap as DM

class App(Tkinter.Frame):
    def __init__(self,master=None):
        Tkinter.Frame.__init__(self,master)
        self.master.title("DNA Diffraction GUI")
        self.errorstr = Tkinter.StringVar()
        self.errorstr.set("Low Resolution Mode.")

        display=Tkinter.Label(self, textvariable = self.errorstr)
        display.pack(side = Tkinter.TOP)
        f0 = Tkinter.LabelFrame(self)
        rd = Tkinter.Label(f0,text="Run No.:")
        rd.pack(pady=0, side = Tkinter.LEFT)
        self.RunNo = Tkinter.Spinbox(f0,from_=1000, to=50000,increment = 1, width=5)
        self.RunNo.delete(0, Tkinter.END)
        self.RunNo.insert(0, '3746')
        self.RunNo.pack(pady=2, side= Tkinter.LEFT)
        f0.pack(padx=5,pady=5, fill=Tkinter.BOTH)
        self.pack()

        f2 = Tkinter.LabelFrame(self, text = 'Parameter')
        f20 = Tkinter.LabelFrame( f2, relief = Tkinter.FLAT)
        rd = Tkinter.Label( f20, text="lcenter ")
        rd.pack(pady=0, side = Tkinter.LEFT)
        self.lam = Tkinter.Spinbox( f20, from_=0, to=99.999, increment=0.001, width=6, format='%6.3f')
        self.lam.delete( 0, Tkinter.END)
        self.lam.insert( 0, '6.321')
        self.lam.pack(pady=0, side = Tkinter.LEFT)
        rd = Tkinter.Label( f20, text="[A] / dl")
        rd.pack(pady=0, side = Tkinter.LEFT)
        self.dl = Tkinter.Spinbox( f20, from_=0, to=9.999, increment=0.001, width=5, format='%5.3f')
        self.dl.delete( 0, Tkinter.END)
        self.dl.insert( 0, '3.000')
        self.dl.pack(pady=0, side = Tkinter.LEFT)
        rd = Tkinter.Label( f20, text="[A]")
        rd.pack(pady=0, side = Tkinter.LEFT)
        f20.pack(padx=5, pady=2, fill = Tkinter.BOTH)

        f21 = Tkinter.LabelFrame(f2, text = 'Select OutPut Type')
        self.patID = 0
        self.v = Tkinter.IntVar()
        self.v.set(0)
        #Output for TOF
        f211 = Tkinter.LabelFrame(f21, relief = Tkinter.FLAT)
        rd = Tkinter.Radiobutton(f211, text = 'TOF', value = 0, variable = self.v)#, command = self.sel_si)
        rd.pack(padx=0, pady=2,side=Tkinter.LEFT,anchor=Tkinter.W)
        rd = Tkinter.Label(f211, text = 'TOF(Min.)')
        rd.pack(padx=2, pady=0, side=Tkinter.LEFT,anchor=Tkinter.W)
        self.TOFmin = Tkinter.Spinbox(f211, from_=0, to = 1000000, increment = 1, width = 7, format='%6.1f')
        self.TOFmin.delete(0, Tkinter.END)
        self.TOFmin.insert(0,'40000.0')
        self.TOFmin.pack(padx=0, pady=0, side=Tkinter.LEFT,anchor=Tkinter.W)
        rd = Tkinter.Label(f211, text = 'TOF(Max.)')
        rd.pack(padx=2, pady=0, side=Tkinter.LEFT,anchor=Tkinter.W)
        self.TOFmax = Tkinter.Spinbox(f211, from_=0, to = 1000000, increment = 1, width = 7, format='%6.1f')
        self.TOFmax.delete(0, Tkinter.END)
        self.TOFmax.insert(0,'80000.0')
        self.TOFmax.pack(padx=0, pady=0, side=Tkinter.LEFT,anchor=Tkinter.W)
        rd = Tkinter.Label(f211, text = 'TOF Bin')
        rd.pack(padx=2, pady=0, side=Tkinter.LEFT,anchor=Tkinter.W)
        self.TOFbin = Tkinter.Spinbox(f211, from_=0, to = 80000, increment = 1, width = 5, format='%6.1f')
        self.TOFbin.delete(0, Tkinter.END)
        self.TOFbin.insert(0,'100.0')
        self.TOFbin.pack(padx=0, pady=0, side=Tkinter.LEFT,anchor=Tkinter.W)
        f211.pack(padx=0,pady=2, fill = Tkinter.BOTH,anchor=Tkinter.W)
        #Output for Q
        f212 = Tkinter.LabelFrame(f21, relief = Tkinter.FLAT)
        rd = Tkinter.Radiobutton(f212, text = 'Q', value = 1, variable = self.v)#, command = self.sel_si)
        rd.pack(padx=0, pady=2, side=Tkinter.LEFT,anchor=Tkinter.W)
        rd = Tkinter.Label(f212, text = 'Q(Min.)')
        rd.pack(padx=2, pady=0, side=Tkinter.LEFT,anchor=Tkinter.W)
        self.Qmin = Tkinter.Spinbox(f212, from_=0, to = 100, increment = 0.001, width = 7, format='%6.3f')
        self.Qmin.delete(0, Tkinter.END)
        self.Qmin.insert(0,'0.00')
        self.Qmin.pack(padx=0, pady=0, side=Tkinter.LEFT,anchor=Tkinter.W)
        rd = Tkinter.Label(f212, text = 'Q(Max.)')
        rd.pack(padx=2, pady=0, side=Tkinter.LEFT,anchor=Tkinter.W)
        self.Qmax = Tkinter.Spinbox(f212, from_=0, to = 100, increment = 0.001, width = 7, format='%6.3f')
        self.Qmax.delete(0, Tkinter.END)
        self.Qmax.insert(0,'6.0')
        self.Qmax.pack(padx=0, pady=0, side=Tkinter.LEFT,anchor=Tkinter.W)
        rd = Tkinter.Label(f212, text = 'Q Bin')
        rd.pack(padx=2, pady=0, side=Tkinter.LEFT,anchor=Tkinter.W)
        self.Qbin = Tkinter.Spinbox(f212, from_=0, to = 100, increment = 0.001, width = 5, format='%6.4f')
        self.Qbin.delete(0, Tkinter.END)
        self.Qbin.insert(0,'0.02')
        self.Qbin.pack(padx=0, pady=0, side=Tkinter.LEFT,anchor=Tkinter.W)
        f212.pack(padx=0,pady=2, fill = Tkinter.BOTH)
        #Output for d
        f213 = Tkinter.LabelFrame(f21, relief = Tkinter.FLAT)
        rd = Tkinter.Radiobutton(f213, text = 'd', value = 2, variable = self.v)#, command = self.sel_si)
        rd.pack(padx=0, pady=2,side=Tkinter.LEFT, anchor=Tkinter.W)
        rd = Tkinter.Label(f213, text = 'd(Min.)')
        rd.pack(padx=2, pady=0, side=Tkinter.LEFT,anchor=Tkinter.W)
        self.dmin = Tkinter.Spinbox(f213, from_=0, to = 100, increment = 0.001, width = 7, format='%6.3f')
        self.dmin.delete(0, Tkinter.END)
        self.dmin.insert(0,'0.00')
        self.dmin.pack(padx=0, pady=0, side=Tkinter.LEFT,anchor=Tkinter.W)
        rd = Tkinter.Label(f213, text = 'd(Max.)')
        rd.pack(padx=2, pady=0, side=Tkinter.LEFT,anchor=Tkinter.W)
        self.dmax = Tkinter.Spinbox(f213, from_=0, to = 100, increment = 0.001, width = 7, format='%6.3f')
        self.dmax.delete(0, Tkinter.END)
        self.dmax.insert(0,'6.0')
        self.dmax.pack(padx=0, pady=0, side=Tkinter.LEFT,anchor=Tkinter.W)
        rd = Tkinter.Label(f213, text = 'd Bin')
        rd.pack(padx=2, pady=0, side=Tkinter.LEFT,anchor=Tkinter.W)
        self.dbin = Tkinter.Spinbox(f213, from_=0, to = 100, increment = 0.001, width = 5, format='%6.4f')
        self.dbin.delete(0, Tkinter.END)
        self.dbin.insert(0,'0.02')
        self.dbin.pack(padx=0, pady=0, side=Tkinter.LEFT,anchor=Tkinter.W)
        f213.pack(padx=0,pady=2, fill = Tkinter.BOTH)
        f21.pack(padx=5, pady=5, fill = Tkinter.BOTH)
        f2.pack(padx=5, pady=5, fill = Tkinter.BOTH)
        #Time Range
        f3 = Tkinter.LabelFrame(self, text = 'time range') 
        f31 = Tkinter.LabelFrame( f3, relief = Tkinter.FLAT)
        rd = Tkinter.Label( f31, text = "StartTime")
        rd.pack(pady=0, side = Tkinter.LEFT)
        self.Sta = Tkinter.Spinbox( f31, from_=0, to=9999999, increment=1, width=7, format='%7.0f')
        self.Sta.pack(pady=0, side = Tkinter.LEFT)
        rd = Tkinter.Label( f31, text = "[sec.]/ EndTime ")
        rd.pack(pady=0, side = Tkinter.LEFT)
        self.End = Tkinter.Spinbox( f31,from_=0,to=9999999,increment=1,width=7,format='%7.0f')
        self.End.delete( 0, Tkinter.END)
        self.End.insert( 0, '9999999')
        self.End.pack(pady=0, side = Tkinter.LEFT)
        rd = Tkinter.Label( f31, text = "[sec.]")
        rd.pack(pady=0, side = Tkinter.LEFT)
        f31.pack(padx=5,pady=2, fill = Tkinter.BOTH)
        f3.pack(padx=5, pady=5, fill = Tkinter.BOTH)

        button1 = Tkinter.Button(self, text = "Launch Data Reduction", command = self.launch)
        button3 = Tkinter.Button(self, text = "Quit", command = self.master.destroy)
        button1.pack(side = Tkinter.LEFT)
        button3.pack(side = Tkinter.RIGHT)
        self.pack()
    
    def GetType(self):
        Type = 2
        if self.v.get() == 1:
            Type = 22
        elif self.v.get() ==2:
            Type = 25

        return Type
    
    def GetParams(self):
        self.params = []
        self.params.append(self.GetType())
        if self.v.get() == 0:
            min_ = float(self.TOFmin.get())
            max_ = float(self.TOFmax.get())
            bin_ = float(self.TOFbin.get())
        elif self.v.get() == 1:
            min_ = float(self.Qmin.get())
            max_ = float(self.Qmax.get())
            bin_ = float(self.Qbin.get())
        elif self.v.get() == 2:
            min_ = float(self.dmin.get())
            max_ = float(self.dmax.get())
            bin_ = float(self.dbin.get())
        
        self.params.append(min_)
        self.params.append(max_)
        self.params.append(bin_)
        self.params.append(float(self.lam.get()))
        self.params.append(float(self.dl.get()))
        print(self.params)

    def launch(self):
        WIorg = "/opt/mlfsoft_dev/python-utsusemi/DNA/ana/xml/WiringInfoGUIBase.xml"
        DIorg = "/opt/mlfsoft_dev/python-utsusemi/DNA/ana/xml/DetectorInfo20140120GUI.xml"
        self.GetParams()
        dgm = DGM.EditWiringInfoForGUI(WIorg,self.params[0],self.params[1],self.params[2],self.params[3],self.params[4],self.params[5])
        WINew = dgm.OutputXml()

        CommentHeader = "Diffraction GUI ->>"
        print(CommentHeader, WINew)
        
        ecm = DB.GetDiffPattern(int(self.RunNo.get()),float(self.Sta.get()),float(self.End.get()),WINew,DIorg)
        DM.DetectMap(ecm)
        

if __name__ == "__main__":
    app = App()
    app.mainloop()
